from .card import SmarTripCard

import csv
import datetime as dt
import re
import requests

ST_USER_AGENT = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:143.0) Gecko/20100101 Firefox/143.0"
BAD_LOGIN_STRINGS = ("Invalid user name or password.", "Your account has been locked.")
LOGIN_URL = "https://smartrip.wmata.com/Account/Login"
ACCOUNT_SUMMARY_URL = "https://smartrip.wmata.com/Account/Summary"
CARD_REGEX = "/Card/Summary/Index/([0-9]+)"


class SmarTrip:
    def __init__(self, username: str, password: str):
        self.username = username
        self.password = password
        self.session = requests.Session()
        # mimic a browser, site rejects default user agent string
        self.session.headers.update({"User-Agent": ST_USER_AGENT})
        self._login()
        self.cards = self._get_available_cards()

    def _get_rvt(self, url: str):
        r = self.session.get(url)
        if r.ok:
            tmp = re.search("[a-zA-Z0-9-_]{92}", r.text)
            if tmp:
                return tmp.group(0)
        return None

    def _post_with_rvt(self, url: str, data: dict):
        rvt = self._get_rvt(url)
        if not rvt:
            raise Exception(f"failed to get request verification token for: {url}")
        data["__RequestVerificationToken"] = rvt
        return self.session.post(url, data=data)

    def _login(self):
        data = {
            "UserName": self.username,
            "Password": self.password,
            "OnCancel": "",
            "log_in": "",
        }
        r = self._post_with_rvt(LOGIN_URL, data=data)
        if r.ok:
            for bls in BAD_LOGIN_STRINGS:
                if bls in r.text:
                    raise Exception("login failed! username or password invalid")
        else:
            raise Exception("login failed! bad response for login")
        return True

    def _get_available_cards(self):
        r = self.session.get(ACCOUNT_SUMMARY_URL)
        if r.ok:
            return [
                SmarTripCard(self, card_id)
                for card_id in re.findall(CARD_REGEX, r.text)
            ]
        return None



from .trip import Trip

import calendar
import datetime as dt
import csv
import json

USAGE_HISTORY_URL = "https://smartrip.wmata.com/Card/UsageReport/Index"
GET_EXCEL_URL = "https://smartrip.wmata.com/Card/UsageReport/GetExcel"


class SmarTripCard:
    def __init__(self, ctx, card_id: str):
        self.ctx = ctx
        self.card_id = card_id

    def __repr__(self):
        return f"SmarTripCard({self.card_id})"

    def __str__(self):
        return f"SmarTripCard({self.card_id})"

    def _get_results(self, params):
        const = {
            "TransactionStatus": "Successful",
            "cardId": self.card_id,
        }
        r = self.ctx.session.get(GET_EXCEL_URL, params=params | const)
        if r.ok:
            return [Trip.from_csv(line) for line in r.text.split("\n")[1:][::-1]]
        return None

    def _get_month_results(self, month: int, year: int):
        date_fmt = "%m/%d/%Y %H:%M:%S"
        last_day = calendar._monthlen(year, month)
        start = dt.datetime(year, month, 1, 0, 0)
        end = dt.datetime(year, month, last_day, 23, 59)
        params = {
            "Month": f"{year:4}{month:02}",
            "StartDate": start.strftime(date_fmt),
            "EndDate": end.strftime(date_fmt),
            "Period": "M",
        }
        return self._get_results(params)

    def _get_between_results(self, start, end):
        date_fmt = "%m/%d/%Y %H:%M:%S"
        params = {
            "StartDate": start.strftime(date_fmt),
            "EndDate": end.strftime(date_fmt),
            "Period": "R",
        }
        return self._get_results(params)

    def get_trips_by_month(self, month: int, year: int):
        data = {
            "IsByMonth": "true",
            "SelectedMonth": f"{year:4}{month:02}",
            "CardId": int(self.card_id),
            "BackUrl": f"~/CardSummary/Index/{self.card_id}",
            # does this matter?
            "MinStartDate": "1/1/1970+12:00:00AM",
            "SubmitButton": "True",
        }
        r = self.ctx._post_with_rvt(f"{USAGE_HISTORY_URL}/{self.card_id}", data)
        if r.ok:
            return self._get_month_results(month, year)
        return None

    def get_trips_between(self, start_date: str, end_date: str):
        start = None
        end = None
        date_fmt = "%m/%d/%Y"
        try:
            start = dt.datetime.strptime(start_date, date_fmt)
            end = dt.datetime.strptime(end_date, date_fmt)
            end += dt.timedelta(hours=23, minutes=59)
        except:
            raise Exception("improperly formatted date (mm/dd/yyyy)")
        data = {
            "IsByMonth": "false",
            "CardId": int(self.card_id),
            "BackUrl": f"~/CardSummary/Index/{self.card_id}",
            "StartDate": start_date,
            "EndDate": end_date,
            # does this matter?
            "MinStartDate": "1/1/1970+12:00:00AM",
            "SubmitButton": "True",
        }
        r = self.ctx._post_with_rvt(f"{USAGE_HISTORY_URL}/{self.card_id}", data)
        if r.ok:
            return self._get_between_results(start, end)
        return None



from datetime import datetime


class Trip:
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def __str__(self):
        return f"Trip({self.seq_no})"

    def __repr__(self):
        return f"Trip({self.seq_no})"

    """
    Seq. #,Time,Description,Operator,Entry Location/ Bus Route,Exit Location,Product,Rem. Rides,Change (+/-),Balance
    """

    @classmethod
    def from_csv(cls, line):
        seq_no = None
        time = None
        description = None
        operator = None
        entry_location = None
        bus_route = None
        exit_location = None
        product = None
        rem_rides = None
        change = None
        balance = None

        for i, v in enumerate(line.split(",")):
            match i:
                case 0:
                    seq_no = int(v)
                case 1:
                    time = datetime.strptime(v, "%m/%d/%y %I:%M %p")
                case 2:
                    description = v
                case 3:
                    operator = v
                case 4:
                    if v:
                        if v[0].isdigit():
                            bus_route = v
                        else:
                            entry_location = v
                case 5:
                    exit_location = v
                case 6:
                    product = v
                case 7:
                    rem_rides = v
                case 8:
                    change = v
                case 9:
                    balance = v

        return cls(
            seq_no=seq_no,
            time=time,
            description=description,
            operator=operator,
            entry_location=entry_location,
            bus_route=bus_route,
            exit_location=exit_location,
            product=product,
            rem_rides=rem_rides,
            change=change,
            balance=balance,
        )
